﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace Artech.ExtendedUltraGrid4SelectAll
{
    public class CheckableHeaderCreationFilter : IUIElementCreationFilter
    {
        public ExtendedUltraGrid Control
        { get; private set; }

        public CheckableHeaderCreationFilter( ExtendedUltraGrid control)
        {          
            if (null == control)
            {
                throw new ArgumentNullException("control");
            }
            this.Control = control;
        }

        public void AfterCreateChildElements(UIElement parent)
        {
            //Filter the HeaderUIElement element.
            HeaderUIElement headerUIElement = parent as HeaderUIElement;
            if (null == headerUIElement)
            {
                return;
            }

            //Filter by DataType and SelectAll column name.
            if (headerUIElement.Header.Column.DataType != typeof(bool) || headerUIElement.Header.Column.Key != this.Control.SelectAllColumnName)
            {
                return;
            }

            //Check the exitence of CheckBoxUIElement.
            //If not, create a new one and wrap it with a WeakReference.
            CheckBoxUIElement checkBoxUIElement = parent.GetDescendant(typeof(CheckBoxUIElement)) as CheckBoxUIElement;
            if (null == checkBoxUIElement)
            {
                checkBoxUIElement = new ExtendedCheckBoxUIElement(parent);              
            }
            //Add the CheckBoxUIElement and set its position.
            parent.ChildElements.Add(checkBoxUIElement);
            checkBoxUIElement.Rect = new Rectangle(
                        parent.Rect.X + (parent.Rect.Width - checkBoxUIElement.CheckSize.Width) / 2 + 1,
                        parent.Rect.Y + ((parent.Rect.Height - checkBoxUIElement.CheckSize.Height) / 2),
                        checkBoxUIElement.CheckSize.Width,
                        checkBoxUIElement.CheckSize.Height
                        );           
            //For GroupRow, set the Tag as the current CheckState.
            UltraGridRow ultraGridRow = headerUIElement.GetContext(typeof(UltraGridRow)) as UltraGridRow;
            UltraGridGroupByRow parentRow = ultraGridRow.ParentRow as UltraGridGroupByRow;
            if (null != parentRow && null != parentRow.Tag)
            {
                checkBoxUIElement.CheckState = (CheckState)parentRow.Tag;
            }

            if (!this.Control.IsGroupMode)
            {
                checkBoxUIElement.CheckState = this.Control.CheckState;
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }
    }
}